"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImportedFile = exports.extractLocalImportContext = exports.getImportedFilesInterface = void 0;
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_file_service_1 = require("@opensumi/ide-file-service");
const path_1 = require("@opensumi/ide-utils/lib/path");
const ide_workspace_1 = require("@opensumi/ide-workspace");
const service_1 = require("../../languages/service");
const languages_1 = require("./languages");
const getImportedFilesInterface = async (code, resource, options, injector) => {
    var _a;
    const languageParserService = injector.get(service_1.LanguageParserService);
    let matches = [];
    const snippets = [];
    const languageParser = languageParserService.createParser(resource.languageId);
    if (languageParser) {
        matches = await languageParser.extractImportPaths(code);
    }
    const maxPromptTime = (_a = options.maxTime) !== null && _a !== void 0 ? _a : 200;
    const startTime = Date.now();
    for (const file of matches) {
        let filePath = '';
        const basePath = resource.uri.parent.codeUri.fsPath;
        if (file.startsWith('@/') || file.startsWith('~') || !file.startsWith('.')) {
            // skip alias
            continue;
        }
        if (file === '.') {
            filePath = new path_1.Path(basePath).join('index.ts').toString();
        }
        else {
            filePath = new path_1.Path(basePath).join(`${file}${languages_1.LANGUAGE_TO_SUFFIX[resource.languageId]}`).toString();
        }
        try {
            const fileService = injector.get(ide_file_service_1.IFileServiceClient);
            const workspaceService = injector.get(ide_workspace_1.IWorkspaceService);
            const fileUri = ide_core_common_1.URI.file(filePath);
            const content = await fileService.readFile(fileUri.codeUri.fsPath);
            const interfaceCode = await (languageParser === null || languageParser === void 0 ? void 0 : languageParser.extractInterfaceOrTypeCode(content.content.toString()));
            if (interfaceCode && interfaceCode.length > 0) {
                const relative = await workspaceService.asRelativePath(fileUri);
                if (relative === null || relative === void 0 ? void 0 : relative.path) {
                    snippets.push([relative.path, interfaceCode]);
                }
            }
        }
        catch (_b) { }
        if (Date.now() - startTime > maxPromptTime) {
            break;
        }
    }
    return snippets;
};
exports.getImportedFilesInterface = getImportedFilesInterface;
const extractLocalImportContext = async (resource, options, injector) => {
    // 从代码中提取出所有的 import 语句，并进一步提取 interface 相关代码声明
    const { source } = resource;
    if (resource.languageId === 'typescript') {
        return (0, exports.getImportedFilesInterface)(source, resource, options, injector);
    }
    return [];
};
exports.extractLocalImportContext = extractLocalImportContext;
const getImportedFile = async (context, options, injector) => {
    if (context.language && !options.language.includes(context.language)) {
        return [];
    }
    const content = context.prefix;
    return (0, exports.extractLocalImportContext)({
        source: content,
        uri: context.uri,
        languageId: context.language,
        offset: content.length,
    }, options, injector);
};
exports.getImportedFile = getImportedFile;
//# sourceMappingURL=importedFiles.js.map