"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FixedWindowSizeJaccardMatcher = exports.getCursorContext = void 0;
const types_1 = require("../types");
const matcher_1 = require("./matcher");
const tokenizer_1 = require("./tokenizer");
const getCursorContext = (resource, option = {}) => {
    const baseOptions = Object.assign({ tokenizerName: types_1.TokenizerName.cl100k_base }, option);
    const tokenizer = (0, tokenizer_1.getTokenizer)(baseOptions.tokenizerName);
    if (baseOptions.maxLineCount && baseOptions.maxLineCount < 0) {
        throw new Error('maxLineCount must be non-negative if defined');
    }
    if (baseOptions.maxTokenLength && baseOptions.maxTokenLength < 0) {
        throw new Error('maxTokenLength must be non-negative if defined');
    }
    if (baseOptions.maxLineCount === 0 || baseOptions.maxTokenLength === 0) {
        return {
            context: '',
            lineCount: 0,
            tokenLength: 0,
            tokenizerName: baseOptions.tokenizerName,
        };
    }
    let context = resource.source.slice(0, resource.offset);
    if (baseOptions.maxLineCount) {
        context = context.split('\n').slice(-baseOptions.maxLineCount).join('\n');
    }
    return {
        context,
        lineCount: context.split('\n').length,
        tokenLength: tokenizer.encode(context).length,
        tokenizerName: baseOptions.tokenizerName,
    };
};
exports.getCursorContext = getCursorContext;
const getBasicWindowDelineations = (windowSize, lineArrays) => {
    const ranges = [];
    const arrayLength = lineArrays.length;
    if (arrayLength === 0) {
        return [];
    }
    if (arrayLength < windowSize) {
        return [[0, arrayLength]];
    }
    for (let i = 0; i < arrayLength - windowSize + 1; i++) {
        ranges.push([i, i + windowSize]);
    }
    return ranges;
};
// 固定窗口大小的 Jaccard 匹配器
class FixedWindowSizeJaccardMatcher extends matcher_1.WindowedMatcher {
    static factory(windowSize) {
        return {
            to: (doc) => new FixedWindowSizeJaccardMatcher(doc, windowSize),
        };
    }
    constructor(doc, windowSize) {
        super(doc);
        this.windowLength = windowSize;
    }
    id() {
        return `fixed:${this.windowLength}`;
    }
    getWindowsDelineations(lines) {
        return getBasicWindowDelineations(this.windowLength, lines);
    }
    getCursorContextInfo(resource) {
        return (0, exports.getCursorContext)(resource, { maxLineCount: this.windowLength });
    }
    similarityScore(set1, set2) {
        return (0, matcher_1.computeScore)(set1, set2);
    }
}
exports.FixedWindowSizeJaccardMatcher = FixedWindowSizeJaccardMatcher;
//# sourceMappingURL=jaccardMatcher.js.map