"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSimilarSnippets = exports.getNeighboringResource = exports.getNeighboringDocument = exports.getRecentEditFileList = exports.getOpenedTabFileList = void 0;
const ide_editor_1 = require("@opensumi/ide-editor");
const ide_workspace_1 = require("@opensumi/ide-workspace");
const utils_1 = require("../../../common/utils");
const types_1 = require("../types");
const const_1 = require("./const");
const jaccardMatcher_1 = require("./jaccardMatcher");
const getOpenedTabFileList = (docuemnts) => {
    // 过滤超大文档
    const recentFiles = docuemnts.filter((document) => (0, utils_1.isDocumentValid)(document));
    return recentFiles;
};
exports.getOpenedTabFileList = getOpenedTabFileList;
const getRecentEditFileList = () => [];
exports.getRecentEditFileList = getRecentEditFileList;
const getNeighboringDocument = async (type, injector) => {
    switch (type) {
        case types_1.NeighboringTabsOption.openFileHistory:
            const editorService = injector.get(ide_editor_1.WorkbenchEditorService);
            const documents = await editorService.getAllOpenedDocuments();
            return (0, exports.getOpenedTabFileList)(documents);
        case types_1.NeighboringTabsOption.editFileHistory:
            // 暂不支持
            return [];
        case types_1.NeighboringTabsOption.editFileRecent:
            return [];
        default:
            return [];
    }
};
exports.getNeighboringDocument = getNeighboringDocument;
const getNeighboringResource = async (targetFileName, languageId, option, injector) => {
    let documentList = [];
    for (const type of option.neighboringTabsOption) {
        documentList = documentList.concat(await (0, exports.getNeighboringDocument)(type, injector));
    }
    documentList = documentList.filter((document) => document.languageId === languageId && !document.uri.displayName.endsWith(targetFileName));
    // 移除重复项
    documentList = documentList.filter((document, index) => documentList.findIndex((item) => item.uri === document.uri) === index);
    let neighboringTabsMaxNum = 20;
    if (typeof option.neighboringTabsMaxNum === 'number') {
        neighboringTabsMaxNum = option.neighboringTabsMaxNum;
    }
    documentList = documentList.slice(0, neighboringTabsMaxNum);
    const resources = documentList.map((document) => {
        const content = document.getText();
        return {
            source: content,
            uri: document.uri,
            languageId: document.languageId,
            offset: content.length,
        };
    });
    return resources;
};
exports.getNeighboringResource = getNeighboringResource;
const getSimilarSnippets = async (context, options, injector) => {
    var _a, _b;
    const editorService = injector.get(ide_editor_1.WorkbenchEditorService);
    const workspaceService = injector.get(ide_workspace_1.IWorkspaceService);
    // 过滤文件，仅保留相同后缀文件
    // 仅保留 20 项，顺序取决于配置顺序
    const neighboringFiles = await (0, exports.getNeighboringResource)(context.filename, context.language, options, injector);
    const snippetLength = (context.prefix + context.suffix).split('\n').length;
    // 窗口大小最大不大于服务端配置的 windowSize 大小
    const windowSize = snippetLength > options.windowSize ? options.windowSize : snippetLength;
    const doMatcher = jaccardMatcher_1.FixedWindowSizeJaccardMatcher.factory(windowSize);
    const bestMatchSnippets = [];
    const currentDocument = (_a = editorService.currentEditor) === null || _a === void 0 ? void 0 : _a.currentDocumentModel;
    if (!currentDocument) {
        return [];
    }
    const content = currentDocument.getText();
    const matcher = doMatcher.to({
        source: content,
        uri: currentDocument.uri,
        languageId: currentDocument.languageId,
        offset: content.length,
    });
    let currentLength = 0;
    const maxPromptTime = (_b = options.maxTime) !== null && _b !== void 0 ? _b : 1000;
    const startTime = Date.now();
    for (const file of neighboringFiles) {
        currentLength += file.source.length;
        const snippet = matcher.findMatches(file, options.snippetSelectionMode);
        if (snippet && snippet.length) {
            const relative = await workspaceService.asRelativePath(file.uri);
            if (relative === null || relative === void 0 ? void 0 : relative.path) {
                bestMatchSnippets.push([relative.path, snippet]);
            }
        }
        if (Date.now() - startTime > maxPromptTime) {
            break;
        }
        // 计算相似性片段总长度不能超过 MAX_NEIGHBOR_AGGREGATE_LENGTH
        if (currentLength > const_1.MAX_NEIGHBOR_AGGREGATE_LENGTH) {
            break;
        }
    }
    // 对相似片段以 score 降序排序，仅取前 snippetMaxNum 项
    const sortedSnippets = bestMatchSnippets
        .filter((item) => { var _a; return item[1][0] && item[1][0].score > ((_a = options.similarityThreshold) !== null && _a !== void 0 ? _a : 0.6); })
        .sort((a, b) => b[1][0].score - a[1][0].score)
        .slice(0, options.snippetMaxNum || 4);
    return sortedSnippets;
};
exports.getSimilarSnippets = getSimilarSnippets;
//# sourceMappingURL=similarSnippets.js.map