"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSuffixPrompt = exports.getPrefixPrompt = void 0;
const importedFiles_1 = require("./prompt/importedFiles");
const prompt_1 = require("./prompt/prompt");
const similarSnippets_1 = require("./prompt/similarSnippets");
async function getPrefixPrompt(context, promptConfig, injector, token) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const beforeCursor = context.prefix;
    const smiilarSnippetPriority = promptConfig.wishList.similarFile.priority;
    const beforeCursorPriority = promptConfig.wishList.beforeCursor.priority;
    const importedFilePriority = promptConfig.wishList.importedFile.priority;
    const pathMarkderPriority = promptConfig.wishList.pathMarker.priority;
    const languageMarkerPriority = promptConfig.wishList.languageMarker.priority;
    const { maxPromptTokenSize } = promptConfig;
    const prefixPercent = (_a = 1 - promptConfig.wishList.afterCursor.extOption.suffixPercent) !== null && _a !== void 0 ? _a : 0.25;
    const beforeCursorMaxTokenSize = Math.ceil(prefixPercent * maxPromptTokenSize);
    const leftTokenSize = beforeCursorMaxTokenSize;
    // Language Marker
    const languageMarker = promptConfig.wishList.languageMarker.enable ? (0, prompt_1.getLanguageMarker)(context.language) : '';
    // Path Marker
    const pathMarker = promptConfig.wishList.pathMarker.enable ? (0, prompt_1.getPathMarker)(context.filename, context.language) : '';
    // Similar Snippet
    const similarFileOptions = Object.assign(Object.assign({}, promptConfig.wishList.similarFile.extOption), { maxTime: Math.min(promptConfig.wishList.similarFile.extOption.maxTime || 200, promptConfig.maxExecuteTimeMillSecond) });
    const now = Date.now();
    const similarFileSnippets = promptConfig.wishList.similarFile.enable
        ? await (0, similarSnippets_1.getSimilarSnippets)(context, similarFileOptions, injector)
        : [];
    if (token === null || token === void 0 ? void 0 : token.isCancellationRequested) {
        return beforeCursor;
    }
    const costTime = Date.now() - now;
    // Imported File
    const importedFileOptions = Object.assign(Object.assign({}, promptConfig.wishList.importedFile.extOption), { maxTime: Math.min(promptConfig.wishList.importedFile.extOption.maxTime, promptConfig.maxExecuteTimeMillSecond - costTime) });
    const importedFiles = promptConfig.wishList.importedFile.enable
        ? await (0, importedFiles_1.getImportedFile)(context, importedFileOptions, injector)
        : [];
    if (token === null || token === void 0 ? void 0 : token.isCancellationRequested) {
        return beforeCursor;
    }
    // 根据内容优先级拼接 prompt
    const promptPriority = [
        {
            attributeName: promptConfig.wishList.languageMarker.attributeName,
            priority: languageMarkerPriority,
            maxPercent: (_b = promptConfig.wishList.languageMarker.extOption.maxPercent) !== null && _b !== void 0 ? _b : 1,
            enable: (_c = promptConfig.wishList.languageMarker.enable) !== null && _c !== void 0 ? _c : false,
            content: languageMarker,
        },
        {
            attributeName: promptConfig.wishList.pathMarker.attributeName,
            priority: pathMarkderPriority,
            maxPercent: (_e = (_d = promptConfig.wishList.pathMarker.extOption) === null || _d === void 0 ? void 0 : _d.maxPercent) !== null && _e !== void 0 ? _e : 1,
            enable: (_f = promptConfig.wishList.pathMarker.enable) !== null && _f !== void 0 ? _f : false,
            content: pathMarker,
        },
        {
            attributeName: promptConfig.wishList.importedFile.attributeName,
            priority: importedFilePriority,
            enable: (_g = promptConfig.wishList.importedFile.enable) !== null && _g !== void 0 ? _g : false,
            maxPercent: (_h = promptConfig.wishList.importedFile.extOption.maxPercent) !== null && _h !== void 0 ? _h : 1,
            importedFiles,
        },
        {
            attributeName: promptConfig.wishList.similarFile.attributeName,
            priority: smiilarSnippetPriority,
            enable: (_j = promptConfig.wishList.similarFile.enable) !== null && _j !== void 0 ? _j : false,
            similarSnippets: similarFileSnippets,
            maxPercent: (_k = promptConfig.wishList.similarFile.extOption.maxPercent) !== null && _k !== void 0 ? _k : 1,
        },
        {
            attributeName: promptConfig.wishList.beforeCursor.attributeName,
            priority: beforeCursorPriority,
            maxPercent: (_l = promptConfig.wishList.beforeCursor.extOption.maxPercent) !== null && _l !== void 0 ? _l : 1,
            enable: (_m = promptConfig.wishList.beforeCursor.enable) !== null && _m !== void 0 ? _m : true,
            content: beforeCursor,
        },
    ];
    return (await (0, prompt_1.getBeforePrompt)(promptPriority, context, promptConfig, leftTokenSize, injector, token)) || beforeCursor;
}
exports.getPrefixPrompt = getPrefixPrompt;
async function getSuffixPrompt(context, promptConfig, injector, token) {
    return (await (0, prompt_1.getAfterPrompt)(context, promptConfig, injector, token)) || '';
}
exports.getSuffixPrompt = getSuffixPrompt;
//# sourceMappingURL=provider.js.map